#version 430 core
#define M_PI 3.1415926535897932384626433832795
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;

uniform float time;
uniform float speedDelta;



float centerOffset = 35;
float diffScale = 0.04;
float spatialFreq = 0.05;
float speed = 4;

void main() {
	vert.normal_world = normalMatrix * normal;
	vert.uv = uv;
    
    //Wiggle Animation
	float zOffset = (position.x - centerOffset) * diffScale * sin(spatialFreq * position.x + (time * speed) + ((speed - (speedDelta * 0.2)) * 2.0));
	vec3 newPos = position + vec3(0, 0, zOffset);

	vec4 position_world_ = modelMatrix * vec4(newPos, 1);
	vert.position_world = position_world_.xyz;

	gl_Position = viewProjMatrix * position_world_;
}