#version 430 core

uniform mat4 modelMatrix;
uniform mat4 modelViewMatrix;
uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform mat3 normalMatrix;
uniform float time;
uniform vec3 color;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
} vert;


vec3 sine(vec3 pos) { 
	return abs(vec3(
	    sin(pos.x + sin(pos.y + sin(pos.z))),
	    sin(pos.z + sin(pos.x + sin(pos.y))),
	    sin(pos.y + sin(pos.z + sin(pos.x)))
    ));
}

float sineNoise(vec3 pos)
{
	mat2 m2 = mat2(0.9,  0.2, 0.05,  0.1);
	float rz = 0.0; //return to zero
	float z = 1.4;
	vec3 bp = pos; //biphasic pulse
	for(float i = 0.0; i < 4.0; i++) {
		vec3 dg = sine(bp * 0.1);
        	pos += (dg + time * 0.1) * 0.95;

         	bp *= 1.0;
         	pos *= 1.2;
         	pos.xy *= m2;
         	pos = pos.yzx;
         	pos *= vec3(4.2, 0.0, 0.0);

            	float c = pos.y + abs(fract(pos.x) - 1.5) + abs(fract(pos.z) + 0.5);
            	float s = abs(fract(c) - 0.5);
        	rz += s / z;
        	bp += 0.14;
	}
	rz = 1.4 / rz;
	rz *= 0.5;
	return rz;
}

void main() {
    float brightness = sineNoise(vert.position_world);
    brightness *= brightness * brightness;

    gl_FragColor = vec4(color * brightness, 1.0);
}